package parMSpindle;
/* Env... static class for program values */

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.util.Date;

import javax.vecmath.Color3f;
import javax.vecmath.Point3d;

import edu.cornell.lassp.houle.RngPack.RanMT;

public class Env { 
//  **** psuedo-random number generator (PRNG) ****
	static final RanMT mtRNG = new RanMT(new Date());
//  **** Useful Strings ***
	static final String kOnUnits = " /microMolar-s";
	static final String kOffUnits = " /s";
	static final String concUnits = " microMolar";
	static final String distUnits = " microns";
	static final String degUnits = " degrees";
	
//  **** Physical Constants ****
	static final double Boltz = 1.380662e-23;			// Boltzman's constant in J/K
	static final double tempK = 298.15;				// 25 deg Celsius in Kelvin
	static final double AvogadroNum = 6.02214199e23;	// Avogadro's number.. #/Mole
	
//	*** The Virtual World Dimensions ****
	static Color3f dumC = new Color3f(1.0f,0f,.2f);
	static final Pt3D worldDimension = new Pt3D(1,1,1);
	static final Pt3D farfarAway = new Pt3D(1000,1000,1000);
	
//	**** Times and Counters
	static private final double deltaT_init = 0.00001;		// seconds
	static private final double biochemDeltaT_init = 0.001;		// seconds
	static private final double collisionDeltaT_init = 0.0001;		// seconds
	static private final double brownianDeltaT_init = deltaT_init*10;
	static double simulationTime = 0;			// keeps track of current time
	static private double runTime_init = 1200.0;				// final time to run to
	static int counter = 0;					// keeps track of current integration step number
	
	static final Parameter deltaT = new Parameter ("deltaT"," Time Step", deltaT_init, "seconds");
	static final Parameter biochemDeltaT =  new Parameter("biochemDeltaT"," Time Step For Biochemical Events", biochemDeltaT_init,"seconds");
	static final Parameter collisionDeltaT = new Parameter ("collisionDeltaT"," Time Step For Collison Detection", collisionDeltaT_init, "seconds");
	static final Parameter brownianDeltaT = new Parameter ("brownianDeltaT"," Time Step at which to apply Brownian forces", brownianDeltaT_init, "seconds");
	static final Parameter runTime = new Parameter ("runTime"," Run Time", runTime_init, "seconds");
	
	
	//	**** "Alberts Force", or Pairwise Approx. Lagrangian Multipliers (PALM) related ****
	static private double fracMove_init = 0.3;		// fraction of the calculated AlbertsForce distance to move in a time-step... < 1 for stability
	static private double fracR_init = 0.3;		// as above, but for torque from link force .. bigger numbers are stiffer / faster response times
	static private double fracMoveTorq_init = 0.02;		// as above, but for torsion springs only.. bigger numbers are stiffer / faster response times
	
	static final Parameter fracMove = new Parameter ("fracMove"," Coeff. for PALM movement",fracMove_init,"");
	static final Parameter fracR = new Parameter ("fracR"," Coeff. torque arm for PALM movement",fracR_init,"");
	static final Parameter fracMoveTorq = new Parameter ("fracMoveTorq"," Coeff. for PALM torque movement",fracMoveTorq_init,"");
	
	//  **** Value Tracking *****
	static final int forcesToTrack = 4;
	static final int plasTetherStrainToAverage = 10;
	static final int filTorqueToAverage = 4;
	static final int filLinkForcesToAve = 5;
	static final int filLinkStrainToAve = 3;
	static final int segSegTorqueToAverage = 4;
	static final int bugPlasmidCollisionsToSum = 20;
	static final int compressionStepsToTrack = 100;
	static final int segDistToTrack = 10;

	//  **** Files and Paths ****
	static String outFileName = null;
	static String logFolderPath = null;
	static String srcFilePath = null;
	static String qkFilePath = null;
	static String fromQKFilePath;
	static String fromQKFileName;
	static String fromQKStatePath;
	static File paramFile = null;
	
//  **** For Synchronization of Methods ****
	static Object safeO = new Object();		// handle for operations that cannot be performed at the same time
	
//  **** Flags and Counters for Graphics ****
	static boolean remote = false;			// flag for no graphics instantiation
	static boolean logFiles = false;			// write info to files
	static boolean paintOn = true;
	static boolean toFile = false;
	static boolean toQKFile = false;
	static boolean fromQKFile = false;
	static boolean loadingQKFile = false;
	static boolean paused = true;
	static boolean viewRotation = false;
	static boolean orderedCentered = false;
	
	//static private boolean biochemOnly_init = false;		// flag for only biochemical simulation... no collisions, forces
	static final Parameter biochemOnly = new Parameter ("biochemOnly"," Biochem. Sim. Only", 0, " ", Parameter.BOOLEAN,false);
	
	static private final int drawInterval_init = 100;
	static private final int toFileInterval_init = 100;
	static private final int toQKFileInterval_init = 100;
	static private final double jpegQuality_init = 1.0;
	static private final int remoteReportInterval_init = (int)1e6;		//time-steps
	static final boolean timeStampJPEGs = true;		// write time and other info on JPEG output
	
	static final Parameter drawInterval = new Parameter ("drawInterval"," Drawing Interval", drawInterval_init, "time steps",Parameter.INT);
	static final Parameter toFileInterval = new Parameter ("toFileInterval"," JPEG Interval", toFileInterval_init, "time steps",Parameter.INT);
	static final Parameter toQKFileInterval = new Parameter ("toQKFileInterval"," QK Interval", toQKFileInterval_init, "time steps",Parameter.INT);
	static final Parameter jpegQuality = new Parameter ("jpegQuality"," JPEG Quality", jpegQuality_init, " (1.0 is best)");
	static final Parameter remoteReportInterval = new Parameter ("remoteWriteInterval", " Remote Reporting Interval",remoteReportInterval_init,"time steps",Parameter.INT);
	
	static String toFilePath;
	static String toFileName;
	//static String toQKFilePath;
	//static String toQKFileName;
	
	// Rendering from QK files
	static boolean filRenderOff = false;
	static int curRenderNum = 0;
	
	//  **** Flags and Constants for Testing ****
	static boolean brownianFilMotionOff = false;
	static boolean plasmidBrownianMotionOff = false;
	static boolean testLinkedPlasmids = false;
	static final double tstPlasmidOffset = 0.2;
	static boolean compressionCritOff = false;
	
//  **** Graphics Sizes Etc ****
	static private final int frameWidth_init = 800;
	static private final int frameHeight_init = 480;
	
	static final Parameter frameWidth = new Parameter ("frameWidth"," Frame Width", frameWidth_init, "Pixels",Parameter.INT);
	static final Parameter frameHeight = new Parameter ("frameHeight"," Frame Height", frameHeight_init, "Pixels",Parameter.INT);
	
	static final int plasmidTessalation = 60;
	static final int parMEndSphereTessalation = 30;
	static final int monomerTessalation = 5;
	
	static boolean helixSpheres = true;
	static boolean filCoordSysOn = false;
	static boolean bugCoordSysOn = false;
	static boolean bugWired = false;
	static boolean bugWiredCoarse = true;
	static boolean infoIn2D = false;
	static boolean infoIn3D = true;
	static boolean plasmidFaces = false;
	
	static final double transScale = 8;
	
	static final int maxLayout = 33;

//	**** Initial Thing Creation ****
	static private final int initialFilaments_init = 1;
	static private final int initialPlasmids_init = 2;
	
	static final Parameter initialFilaments = new Parameter ("initialFilaments"," Initial ParM Filaments", initialFilaments_init," ",Parameter.INT);
	static final Parameter initialPlasmids = new Parameter ("initialPlasmids"," Initial Plasmids", initialPlasmids_init," ",Parameter.INT);
	
	
//  **** The Bug ****
	static boolean bugOff = false;
	static private final double bugLength_init = 3.0;		// (microns) length from tip to tip
	static private final double bugRadius_init = 0.5;		// (microns) radius of cylinder and hemispherical caps
	static private final double aeta_init = 0.1;			// viscosity in bug.... Pa-s (1e-3 is water)
	static private final double parMConc_init = 15;			// M concentration of parM monomers
	static private final double parMConcNonHydro_init = 15;	// M concentration of non-hydrolyzable parM monomers
	
	static final Parameter bugLength = new Parameter ("bugLength"," Bug Length", bugLength_init, distUnits);
	static final Parameter bugRadius = new Parameter ("bugRadius"," Bug Radius", bugRadius_init, distUnits);
	static final Parameter parMConc = new Parameter ("parMConc"," ParM Concentration", parMConc_init, concUnits, Parameter.DOUBLE,Parameter.CHANGE_VALUE);
	static final Parameter parMConcNonHydro = new Parameter ("parMConcNonHydro"," Non-hydrolyzable ParM Concentration", parMConcNonHydro_init, concUnits, Parameter.DOUBLE,Parameter.CHANGE_VALUE);
	static final Parameter aeta = new Parameter ("aeta"," Viscosity in Bug", aeta_init, "Pa-s (1e-3 is water)");
	
//   **** The Plasmids ****
	static private final double plasmidRadius_init = 0.04;	// microns
	static final Parameter plasmidRadius = new Parameter ("plasmidRadius"," Plasmid Radius", plasmidRadius_init, distUnits);
	
	static private final int filsPerPlasmid_init = 1;		// # of filaments a plasmid can bind
	static final Parameter filsPerPlasmid = new Parameter ("filsPerPlasmid"," Filaments Per Plasmid", filsPerPlasmid_init, "", Parameter.INT);
	
	static private final double plasmidTransDiff_init = Plasmid.defaultTransDiff();
	static final Parameter plasmidTransDiff = new Parameter ("plasmidTransDiff"," Plasmid Translation Diffusivity", plasmidTransDiff_init, "m^2/s",Parameter.DOUBLE);
	
	static private final double plasmidRotDiff_init = Plasmid.defaultRotDiff();
	static final Parameter plasmidRotDiff = new Parameter ("plasmidRotDiff"," Plasmid Rotational Diffusivity", plasmidRotDiff_init, "/s",Parameter.DOUBLE);
	
	static final double plasmidCollisionAttn_init = 0.4;
	static final Parameter plasmidCollisionAttn = new Parameter ("plasmidCollisionAttn"," PAIRS Collision Force Attenuation", plasmidCollisionAttn_init," ",Parameter.DOUBLE);
	
//	 **** ParM Filaments Geometry ****
	static final double parMMonoSize=0.0049;					// (microns)	
	static final double parMHalfMonoSize=parMMonoSize/2.0;		// (microns)
	static private final int parMToSeed_init = 3;
	static final double parMRadius = 0.007;			// (m) radius of parM filament
	static final double helixWaveLength = 0.030;  	// (m) length for one turn of helical filament
	static final double monsPerHelixTurn = helixWaveLength/parMHalfMonoSize;
	static final double helixAngInc = 2*Math.PI/Env.monsPerHelixTurn;
	static final double helixPitch = 2*Math.PI/helixWaveLength;
	static final double helixMonOffset = (parMRadius-parMMonoSize)/2;
	
	static final Parameter parMSeed = new Parameter("parMSeed"," Monomers to seed filament",parMToSeed_init,"",Parameter.INT);
	
	//  **** ParR-ParC ***
	static final double parRparCRadius = 2*parMRadius;
	
	//	 **** ParM Mechanics ****
	static final double persistenceLength = 15
	;	// in microns, use meters in defining EI below
	static final double EI = Boltz*tempK*(persistenceLength*1e-6);		// EI = kTLp from worm-like chain theory
	
	static private final int stdSegLength_init = 100;							// number of monomers per segment
	static final Parameter stdSegLength = new Parameter ("filSegLength"," Std. segment length",stdSegLength_init," monomers",Parameter.INT);
	
	static private final double filTorqSpring_init = 1e-20;					// N/radian   If using elastic springs for torque to keep filaments aligned
	static final Parameter filTorqSpring = new Parameter ("filTorqSpring"," Filament Torsional Spring",filTorqSpring_init," N/radian",Parameter.DOUBLE,false);
	
	static private double BTransCoeff_init = 1;		// range from zero (no brownian motion) to infinity
	static final Parameter BTransCoeff = new Parameter ("BTransCoeff"," Brownian Translational Coefficient",BTransCoeff_init,"");
	
	static private double BRotCoeff_init = 1;		// range from zero (no brownian motion) to infinity
	static final Parameter BRotCoeff = new Parameter ("BRotCoeff"," Brownian Rotational Coefficient",BRotCoeff_init,"");
	
	static private final double maxSegAngle_init = 22.5;	// degrees
	static final Parameter maxSegAngle = new Parameter ("maxSegAngle"," Max. Angle Between Segments",maxSegAngle_init," degrees",Parameter.DOUBLE,false );
	
	static private final double maxSegDist_init = 2*Env.parMMonoSize;
	static final Parameter maxSegDist = new Parameter ("maxSegDist"," Max. Dist. Between Segment EndPts",maxSegDist_init,distUnits,Parameter.DOUBLE,false );
	
	//  **** Attachment to plasmid
	static final Parameter end1CanBindPlasmid = new Parameter ("end1CanBindPlasmid"," Can end1 of segment bind plasmid?",0," ",Parameter.BOOLEAN,true);
	static final Parameter end2CanBindPlasmid = new Parameter ("end2CanBindPlasmid"," Can end2 of segment bind plasmid?",0," ",Parameter.BOOLEAN,true);
	
	static private final double plasmidTorqSpring_init = 1e-20;				// N/radian  If using elastic spring to keep filament orthogonal to plasmid surface
	static final Parameter plasmidTorqSpring = new Parameter ("plasmidTorqSpring"," Plasmid-Filament Torsional Spring",plasmidTorqSpring_init,"N/radian",Parameter.DOUBLE);
	
	static private final double maxPlasTetherStrainDist_init = 2*parMMonoSize;
	static final Parameter maxPlasTetherStrainDist = new Parameter ("maxPlasTetherStrainDist"," Max. Strain Distance in Plasmid Tether",maxPlasTetherStrainDist_init,"microns",Parameter.DOUBLE,false);
	
	static private final double plasTetherDetachRate_init = 0.001;
	static final Parameter plasTetherDetachRate = new Parameter ("plasTetherDetachRate"," Plasmid Tether Detachment Rate",plasTetherDetachRate_init,"/s",Parameter.DOUBLE,false);
	
	static private final double plasCompressDetachFrac_init = 0.5;
	static final Parameter plasCompressDetachFrac = new Parameter ("plasCompressDetachFrac"," Plasmid Compression Detach Fraction",plasCompressDetachFrac_init," ",Parameter.DOUBLE,false);
	
	static final Parameter parRparCMoves = new Parameter ("parRparCMoves"," Mobile Attachment Point to Plasmid",0," ",Parameter.BOOLEAN,false);
	static final double plasmidBoundTransScale = 0.1;
	
	//  ***** ParM Annealing ****
	static final Parameter filamentsAnneal = new Parameter ("filamentsAnneal"," Do filaments end-to-end anneal?",0,"",Parameter.BOOLEAN,true);
	static final double annealProb = 1.0;//0.01;
	static private final double annealDist_init = 10*parMMonoSize;
	static final Parameter annealDist = new Parameter ("annealDist"," Tip distance for annealing",annealDist_init,distUnits);
	
	static private final double annealAngleCosine_init = 0.9;
	static final Parameter annealAngleCosine = new Parameter ("annealAngleCosine"," Cosine of max. angle for annealing",annealAngleCosine_init," unitless");
	
	// biochem annealing and fragmentation.  Annealing constant from Andrianantoandro et al 2001
	static final Parameter annealingConstant = new Parameter ("annealingConstant"," Annealing rate in bioChemOnly sims",1.1,"/mM-s",Parameter.DOUBLE,false);
	static final Parameter fragRate = new Parameter ("fragRate"," Fragmentation rate per micron length",0.0,"/micron-s",Parameter.DOUBLE,false);
	static double oneRateAnnealing = 0;  // set during runtime... is a function of annealingConstant and ave fil length
	
	//	**** Inter ParM filament Mechanics ****
	static final Parameter sideBonds = new Parameter ("sideBonds"," Side bonds?",0,"",Parameter.BOOLEAN,true);
	static final Parameter maxLinksOnSeg = new Parameter ("maxLinksOnSeg"," Max. Cross-Links per Segment",10,"");
	static final Parameter maxToLinkAngle = new Parameter ("maxToLinkAngle"," Max. angle for which to establish link",30,"");
	static final Parameter bindOppositePolarity = new Parameter("bindOppositePolarity"," Bind opposite polarity segments?",0,"",Parameter.BOOLEAN,true);
	static final Parameter bindSamePolarity = new Parameter("bindSamePolarity"," Bind same polarity segments?",0,"",Parameter.BOOLEAN,true);
	static final Parameter maxSegsToBind = new Parameter("maxSegToBind"," Bind no more than this many different segs",1,"",Parameter.INT);
	static final Parameter sideBondsRelax = new Parameter ("sideBondsRelax"," Do side bonds relax, allowing translation of bound fils?",0,"",Parameter.BOOLEAN,true);
	static final Parameter sideBondsStabilize = new Parameter ("sideBondsStabilize"," Do side bonds stabilize depolymerization to this rate?",3.0,"/s",Parameter.DOUBLE,false);
	
	static private final double filLinkSpring_init = 1e-9;					// N/radian   elastic springs link filsegs from different filaments
	static final Parameter filLinkSpring = new Parameter ("filLinkSpring"," Link Spring Between Filaments",filLinkSpring_init," N/m",Parameter.DOUBLE,true);
	
	static private final double filLinkTorqSpring_init = 1e-19;					// N/radian   elastic torque spring links filsegs from different filaments
	static final Parameter filLinkTorqSpring = new Parameter ("filLinkTorqSpring"," Torque Spring Between Filaments",filLinkTorqSpring_init," N/rad",Parameter.DOUBLE,true);
	
	static private double xLinkTransAttn_init = 1;		// range from zero (no attn) to infinity
	static final Parameter xLinkTransAttn = new Parameter ("xLinkTransAttn"," Trans. Attn. Factor per link for Cross-linked Segments",xLinkTransAttn_init,"");
	
	static private double xLinkRotAttn_init = 1;		// range from zero (no attn) to infinity
	static final Parameter xLinkRotAttn = new Parameter ("xLinkRotAttn"," Rot. Attn. Factor per link for Cross-linked Segments",xLinkRotAttn_init,"");
	
	static private final double maxFilLinkDist_init = 10*Env.parMMonoSize;
	static final Parameter maxFilLinkDist = new Parameter("maxFilLinkDist"," Max. Stretch Distance for Filament Links",maxFilLinkDist_init,distUnits,Parameter.DOUBLE,true);
	
	static private final double linkDissolveRate_init = 1.0;  // per second
	static final Parameter linkDissolveRate = new Parameter("linkDissolveRate"," Spontaneous Link Dissolution Rate",linkDissolveRate_init,kOffUnits,Parameter.DOUBLE,true);
	

	//   **** ParM Biochemical Rates ****
	//** End1
	static boolean endsAreSame = false;
	
	static private final double kATPOn1_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff1_init = 0.0;						// s^-1
	//static private final double kADPOn1_init = 0.0;						// M^-1 s^-1
	static private final double kADPOff1_init = 0.0;						// s^-1
	
	static final Parameter kATPOn1 = new Parameter ("kATPOn1"," ATP ParM On Rate", kATPOn1_init, kOnUnits);
	static final Parameter kATPOff1 = new Parameter ("kATPOff1"," ATP ParM Off Rate", kATPOff1_init, kOffUnits);
	//static final Parameter kADPOn1 = new Parameter (" ADP ParM On Rate", kADPOn1_init, kOnUnits);
	static final Parameter kADPOff1 = new Parameter ("kADPOff1"," ADP ParM Off Rate", kADPOff1_init, kOffUnits);
	
	//	** If End Attached to Plasmid
	static private final double kATPOn1WithPlas_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff1WithPlas_init = 0.0;						// s^-1
	//static private final double kADPOn1WithPlas_init = 0.0;					// M^-1 s^-1
	static private final double kADPOff1WithPlas_init = 0.0;						// s^-1
	
	static final Parameter kATPOn1WithPlas = new Parameter ("kATPOn1WithPlas"," ATP ParM On Rate If Attached To Plasmid", kATPOn1WithPlas_init, kOnUnits);
	static final Parameter kATPOff1WithPlas = new Parameter ("kATPOff1WithPlas"," ATP ParM Off Rate If Attached To Plasmid", kATPOff1WithPlas_init, kOffUnits);
	//static final Parameter kADPOn1WithPlas = new Parameter ("kADPOn1WithPlas"," ADP ParM On Rate If Attached To Plasmid", kADPOn1WithPlas_init, kOnUnits);
	static final Parameter kADPOff1WithPlas = new Parameter ("kADPOff1WithPlas"," ADP ParM Off Rate If Attached To Plasmid", kADPOff1WithPlas_init, kOffUnits);
	
	//	** For Non-hydrolyzable ParM
	static private final double kATPOn1NonHydro_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff1NonHydro_init = 0.0;						// s^-1
	
	static final Parameter kATPOn1NonHydro = new Parameter ("kATPOn1NonHydro"," ATP ParM On Rate - Non-hydrolyzable ParM", kATPOn1NonHydro_init, kOnUnits);
	static final Parameter kATPOff1NonHydro = new Parameter ("kATPOff1NonHydro"," ATP ParM Off Rate - Non-hydrolyzable ParM", kATPOff1NonHydro_init, kOffUnits);
	
	//	** For Non-hydrolyzable ParM when attached to Plasmid
	static private final double kATPOn1NonHydroWithPlas_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff1NonHydroWithPlas_init = 0.0;						// s^-1
	
	static final Parameter kATPOn1NonHydroWithPlas = new Parameter ("kATPOn1NonHydroWithPlas"," ATP ParM On Rate - No-hydro/plasmid attached", kATPOn1NonHydroWithPlas_init, kOnUnits);
	static final Parameter kATPOff1NonHydroWithPlas = new Parameter ("kATPOff1NonHydroWithPlas"," ATP ParM Off Rate - No-hydro/plasmid attached", kATPOff1NonHydroWithPlas_init, kOffUnits);

	
	//** End2
	static private final double kATPOn2_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff2_init = 0.0;						// s^-1
	//static private final double kADPOn2_init = 0.0;						// M^-1 s^-1
	static private final double kADPOff2_init = 0.0;						// s^-1
	
	static final Parameter kATPOn2 = new Parameter ("kATPOn2"," ATP ParM On Rate", kATPOn2_init, kOnUnits);
	static final Parameter kATPOff2 = new Parameter ("kATPOff2"," ATP ParM Off Rate", kATPOff2_init, kOffUnits);
	//static final Parameter kADPOn2 = new Parameter (" ADP ParM On Rate", kADPOn2_init, kOnUnits);
	static final Parameter kADPOff2 = new Parameter ("kADPOff2"," ADP ParM Off Rate", kADPOff2_init, kOffUnits);
	
	//** If End Attached to Plasmid
	static private final double kATPOn2WithPlas_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff2WithPlas_init = 0.0;						// s^-1
	//static private final double kADPOn2WithPlas_init = 0.0;					// M^-1 s^-1
	static private final double kADPOff2WithPlas_init = 0.0;						// s^-1
	
	static final Parameter kATPOn2WithPlas = new Parameter ("kATPOn2WithPlas"," ATP ParM On Rate If Attached To Plasmid", kATPOn2WithPlas_init, kOnUnits);
	static final Parameter kATPOff2WithPlas = new Parameter ("kATPOff2WithPlas"," ATP ParM Off Rate If Attached To Plasmid", kATPOff2WithPlas_init, kOffUnits);
	//static final Parameter kADPOn2WithPlas = new Parameter ("kADPOn2WithPlas"," ADP ParM On Rate If Attached To Plasmid", kADPOn2WithPlas_init, kOnUnits);
	static final Parameter kADPOff2WithPlas = new Parameter ("kADPOff2WithPlas"," ADP ParM Off Rate If Attached To Plasmid", kADPOff2WithPlas_init, kOffUnits);
	
	//	** For Non-hydrolyzable ParM
	static private final double kATPOn2NonHydro_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff2NonHydro_init = 0.0;						// s^-1
	
	static final Parameter kATPOn2NonHydro = new Parameter ("kATPOn2NonHydro"," ATP ParM On Rate - Non-hydrolyzable ParM", kATPOn2NonHydro_init, kOnUnits);
	static final Parameter kATPOff2NonHydro = new Parameter ("kATPOff2NonHydro"," ATP ParM Off Rate - Non-hydrolyzable ParM", kATPOff2NonHydro_init, kOffUnits);
	
	//	** For Non-hydrolyzable ParM when attached to Plasmid
	static private final double kATPOn2NonHydroWithPlas_init = 0.0;						// M^-1 s^-1
	static private final double kATPOff2NonHydroWithPlas_init = 0.0;						// s^-1
	
	static final Parameter kATPOn2NonHydroWithPlas = new Parameter ("kATPOn2NonHydroWithPlas"," ATP ParM On Rate - No-hydro/plasmid attached", kATPOn2NonHydroWithPlas_init, kOnUnits);
	static final Parameter kATPOff2NonHydroWithPlas = new Parameter ("kATPOff2NonHydroWithPlas"," ATP ParM Off Rate - No-hydro/plasmid attached", kATPOff2NonHydroWithPlas_init, kOffUnits);

	//** Polymerization Criteria
	static private final int capNumEnd1_init = 3;
	static final Parameter capNumEnd1 = new Parameter ("capNumEnd1"," Cap at End1", capNumEnd1_init, "atp monomers",Parameter.INT);
	
	static private final int capNumEnd2_init = 3;
	static final Parameter capNumEnd2 = new Parameter ("capNumEnd2"," Cap at End2", capNumEnd2_init, "atp monomers",Parameter.INT);
	
	static private final int capNumEnd1WithPlas_init = 3;
	static final Parameter capNumEnd1WithPlas = new Parameter ("capNumEnd1WithPlas"," Cap at End1 with plasmid", capNumEnd1WithPlas_init, "atp monomers",Parameter.INT);
	
	static private final int capNumEnd2WithPlas_init = 3;
	static final Parameter capNumEnd2WithPlas = new Parameter ("capNumEnd2WithPlas"," Cap at End2 with plasmid", capNumEnd2WithPlas_init, "atp monomers",Parameter.INT);
	
	static  final Parameter polyCompressionCutoff = new Parameter ("polyCompressionCutoff"," Compressive force that stops poly.",0, "N (<= 0)");
	static final Parameter noCapThenADPdepoly = new Parameter ("noCapThenADPdepoly"," If no cap then all depoly like ADP", 0, " ",Parameter.BOOLEAN,false);
	
	//** Nucleation
	static private final double kRdmNuc_init = 0.0;						// s^-1
	static final Parameter kRdmNuc = new Parameter ("kRdmNuc"," Random ParM Nucleation Rate", kRdmNuc_init, kOffUnits,Parameter.DOUBLE);
	
	static private final double kPlasmidNuc_init = 0.0;
	static final Parameter kPlasmidNuc = new Parameter ("kPlasmidNuc"," On Plasmid ParM Nucleation Rate", kPlasmidNuc_init, "/Plasmid-s",Parameter.DOUBLE,false);
	
	static private final int maxFils_init = (int)1e6;
	static final Parameter maxFils = new Parameter ("maxFils"," Maximum Filaments (to speed runs)", maxFils_init, "filaments", Parameter.INT);
	
	//** Hydrolysis
	static private final double kHydro_init = 0.2;
	static private final double kDissoc_init = 1000;//0.6;
	static private final double kCoopHydro_init = 2.0;	// multiplies the hydrolysis probability if neighbor is hydrolyzed (e.g. value of 2 makes hydrolysis 2x as likely
	
	static final Parameter kHydrolysis = new Parameter ("kHydrolysis"," Monomer Hydrolysis Rate", kHydro_init, kOffUnits);
	static final Parameter kDissociation = new Parameter ("kDissociation"," Monomer Dissociation Rate", kDissoc_init, kOffUnits);
	
	static final Parameter kTerminalHydro = new Parameter ("kTerminalHydro"," Hydrolysis Rate Just for Terminal Monomers", kHydro_init," ",Parameter.DOUBLE,false);
	static final Parameter kCoopHydro = new Parameter ("kCoopHydro"," Cooperative Hydrolysis Factor", kCoopHydro_init," ",Parameter.DOUBLE,false);
	
//   **** Show Stats Parameters ****
	static final Parameter showTime = new Parameter ("showTime"," Show Time",0,"",Parameter.BOOLEAN);
	static final Parameter showConc = new Parameter ("showConc"," Show ParM Conc",0,"",Parameter.BOOLEAN);
	static final Parameter showNonHydroConc = new Parameter ("showNonHydroConc"," Show Non-Hydro ParM Conc",0,"",Parameter.BOOLEAN,false);
	static final Parameter showFilCt = new Parameter ("showFilCt"," Show Filament Count",0,"",Parameter.BOOLEAN,false);
	static final Parameter showFilLinkCt = new Parameter ("showFilLinkCt"," Show FilSegment Link Count",0,"",Parameter.BOOLEAN,false);
	static final Parameter showFilSegCt = new Parameter ("showFilSegCt"," Show FilSegment Count",0,"",Parameter.BOOLEAN,false);
	static final Parameter showMonCt = new Parameter ("showMonCt"," Show Monomer Count",0,"",Parameter.BOOLEAN);
	
//	 **** Filament Length Histogram Parameters ****
	static private final double rStart_init = 0;	// (micron) start of range for tracking filament lengths
	static private final double rStop_init = 1;	// (micron) end of range for tracking filament lengths
	static private final int binCt_init = 100;		// number of bins in histogram
	static private final int histWidth_init = 200;
	static private final int histHeight_init = 200;
	static private final int histX_init = 0;
	static private final int histY_init = 0;
	static private final int fixedHistScale_init = 100;	// default value using fixed vertical scale in filament length histogram
	
	static final Parameter makeFilLengthHisto = new Parameter ("makeFilLengthHisto"," Filament Length Histogram?",0,"",Parameter.BOOLEAN,false);
	static final Parameter mapMonomerCtToHisto = new Parameter ("mapMonomerCtToHisto"," Map Monomer Count Instead",0,"",Parameter.BOOLEAN);
	static final Parameter minLengthToCount = new Parameter ("minLengthToCount"," Minimum Filament Length To Count",0.0," microns",Parameter.DOUBLE,false);
	static final Parameter rStart = new Parameter ("rStart"," Starting Filament Length",rStart_init,distUnits);
	static final Parameter rStop = new Parameter ("rStop"," Ending Filament Length",rStop_init,distUnits);
	static final Parameter binCt = new Parameter ("binCt"," Bins in Histogram",binCt_init,"",Parameter.INT);
	static final Parameter fixedHistScale = new Parameter ("fixedHistScale"," Fixed Vertical Scale",fixedHistScale_init," filaments",Parameter.DOUBLE,false);
	static final Parameter keepHeads = new Parameter ("keepHeads"," Low Values in First Bin",0,"",Parameter.BOOLEAN);
	static final Parameter keepTails = new Parameter ("keepTails"," High Values in Last Bin",0,"",Parameter.BOOLEAN);
	static final Parameter histWidth = new Parameter ("histWidth"," Width",histWidth_init,"pixels",Parameter.INT);
	static final Parameter histHeight = new Parameter ("histHeight"," Height",histHeight_init,"pixels",Parameter.INT);
	static final Parameter histX = new Parameter ("histX"," X-Position",histX_init,"pixels",Parameter.INT);
	static final Parameter histY = new Parameter ("histY"," Y-Position",histY_init,"pixels",Parameter.INT);
	static final Parameter showHistBox = new Parameter ("showHistBox"," Show Box",0,"",Parameter.BOOLEAN);
	static final Parameter showYHistTics = new Parameter ("showYHistTics"," Show Verical Scale",0,"",Parameter.BOOLEAN);
	static final Parameter flatTops = new Parameter ("flatTops"," Flat Tops on Bins",0,"",Parameter.BOOLEAN);
	
	// for filament lifetimes
	static final Parameter filLifetimeStart = new Parameter ("filLifetimeStart"," Start filament lifetime tracking",120,"seconds",Parameter.DOUBLE);
	static final Parameter minLife = new Parameter ("minLife"," Min. Filament Lifetime",0,"seconds");
	static final Parameter maxLife = new Parameter ("maxLife"," Max. Filament Lifetime",60,"seconds");
	static final Parameter lifeBins = new Parameter ("lifeBins"," Bins in Lifetime Histogram",60,"",Parameter.INT);
	
	// for filament cap states
	static final Parameter minCapState = new Parameter ("minCapState"," Min. Filament Cap",0,"atp monomers");
	static final Parameter maxCapState = new Parameter ("maxCapState"," Max. Filament Cap",60,"atp monomers");
	static final Parameter capStateBins = new Parameter ("capStateBins"," Bins in Cap State Histogram",60,"",Parameter.INT);
	
	// for persistence length tuning
	static boolean testingPersistenceLength = false;
	static int ckPersistenceInterval = (int)2e5; 
	static int persistenceValsToTrack = 100000;
	static double persistenceTstLengthInit = 3.2;	// microns.... approximate desired length
	static int persistenceSegNumber = (int)(persistenceTstLengthInit/(Env.stdSegLength.getValue()*Env.parMHalfMonoSize)); 
	static double persistenceTstLength = persistenceSegNumber*Env.stdSegLength.getValue()*Env.parMHalfMonoSize;
	static ValueTracker [] persistenceTrack;
	static ValueTracker transDiffTrack = new ValueTracker(persistenceValsToTrack,ValueTracker.PT3D_TYPE);
	static Pt3D lastPos = new Pt3D();
	
	// for elasticity tuning
	static boolean cantileverElasticityTst = false;
	static boolean simpleSupportElasticityTst = false;
	static boolean secondMode = false;	
	static boolean relaxCantilever = false;
	static boolean relaxSimpleSupport = false;
	static int ckElasticityInterval = (int)1e6;  // 1 seconds with deltaT of 1e-5
	static double ckElasticitySmallChange = 0.0001;  // .1 nm ... if value hasn't changed by 1 nm over ckElasticityInterval. then we have the value
	static double initDeltaE = 1e-19;
	static double curDeltaE = initDeltaE;
	
	static boolean lastEChangePlus = true;
	static double valOKChange = 0.0001;
	static double tstFilLengthInit = 3.2;//3.2; // microns	This is the approximate desired starting length
	static double tstFilLengthMax = 3.0; // microns   	the approximate end length
	static int curMultipleOfSeg; 
	static int forceSeg1,forceSeg2;		// seg filArrayPos for application of force
	static double tstFilLength; // = (curMultipleOfSeg*Env.stdSegLength.getValue()+1)*Env.parMHalfMonoSize; // microns
	static double desiredDeflection = -0.1;  // microns
	static double forceToGetDesiredDeflection;// = 3*EI*(desiredDeflection*1e-6)/Math.pow(tstFilLength*1e-6,3);
	static Pt3D staticForce = new Pt3D (0,0,0);	// force only applied in y-direction
	static Pt3D staticForceR = new Pt3D (0,0,0);
	static Pt3D rToCross = new Pt3D ();					// used in elasticity torque calc.
	static double expectedDeflection; // = (staticForce.y*Math.pow(tstFilLength*1e-6,3)/(3*EI))*1e6;	// in microns.. from beam theory y=FL^3/(3EI)
	static double relaxationDeflection;
	static double currentDeflection = 0;
	static double lastDeflection = 0;
	static double curDeltaD = 0;
	static double curDistFromExpected = 0;
	static double lastDistFromExpected = 1e6;
	static String elasReportString = " ";
	static int relaxWriteInterval = (int)1e2;
	static int relaxWriteCounter = relaxWriteInterval;  // so it'll write right away when supports relaxed
	static boolean reportedTimeConstant = false;
	
//	 **** 3D Quality ****
	static int graphicsQuality = 0;	// specify graphics quality
	static final int LOW_QUALITY = 0;
	static final int	MID_QUALITY = 1;
	static final int HIGH_QUALITY = 2;
	
	static final int lowSphereTessalation = 10;	// number of polygons to define a sphere
	static final int midSphereTessalation = 18;
	static final int highSphereTessalation = 24;
	
	static final int lowCylXTessalation = 10;		// x and y resolution for cylinders
	static final int lowCylYTessalation = 10;
	static final int midCylXTessalation = 18;
	static final int midCylYTessalation = 18;
	static final int highCylXTessalation = 24;
	static final int highCylYTessalation = 24;
	
	// filament appearance
	static boolean filamentLines = true;
	static boolean filamentCyls = false;
	
	// monomer appearance
	static boolean eachMonomerASphere = false;
	static boolean eachMonomerADot = false;
	
//	 **** GUI AND COLOR RELATED ****
	/* // gray colors
	static final Color3f universeColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f headlightColor = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f cellColor3f = new Color3f(0.1f,0.1f,0.1f);
	static final Color3f plasmidColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f parRparCColor3f = new Color3f(0f,1.0f,1.0f);
	static final Color cellColor = cellColor3f.get();
	static final Color cellShadeColor = Color.BLACK;
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Font elasticityFont = new Font(null,Font.BOLD,12);
	static final Color3f controlBackColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = Color.BLACK;//Color.WHITE;
	static final Color controlFlashColor = Color.WHITE;
	static final Color canvasTextColor = new Color(0.0f,0.0f,0.0f);
	static final Color3f bugEndLightColor3f = new Color3f(0.8f,0.8f,0.8f);
	static final Color3f plasmidLightColor3f = new Color3f(0.0f,0.5f,1.0f);*/
	
	/*// white colors
	static final Color3f universeColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f cellColor3f = new Color3f(0.1f,0.1f,0.1f);
	static final Color3f plasmidColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f parRparCColor3f = new Color3f(0f,1.0f,1.0f);
	static final Color cellColor = cellColor3f.get();
	static final Color cellShadeColor = Color.BLACK;
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Font buttonFont = new Font(null,Font.BOLD,14);
	static final Font elasticityFont = new Font(null,Font.BOLD,12);
	static final Color3f controlBackColor3f = new Color3f(0.6f,0.6f,0.6f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = Color.BLACK;//Color.WHITE;
	static final Color buttonForeColor = Color.BLUE;
	static final Color buttonBackColor = controlBackColor;
	static final Color controlFlashColor = Color.WHITE;
	static final Color canvasTextColor = new Color(0.0f,0.0f,0.0f);
	static final Color3f headlightColor = new Color3f(0.8f,0.9f,0.8f);
	static final Color3f spotLightColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f bugEndLightColor3f = new Color3f(0.8f,0.8f,0.8f);
	//static final Color3f plasmidLightColor3f = new Color3f(0.0f,0.4f,1.0f);
	static final Color3f plasmidLightColor3f = new Color3f(1.0f,1.0f,1.0f);
	*/
	
	// black colors
	static final Color3f universeColor3f = new Color3f(0f,0f,0f);//new Color3f(1.0f,1.0f,1.0f);
	static final Color universeColor = universeColor3f.get();
	static final Color3f headlightColor = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f cellColor3f = new Color3f(0.5f,0.5f,0.5f);
	static final Color3f plasmidColor3f = new Color3f(1.0f,1.0f,1.0f);
	static final Color3f parRparCColor3f = new Color3f(0.0f,1.0f,1.0f);
	static final Font controlFont = new Font(null,Font.PLAIN,10);
	static final Font headFont = new Font(null,Font.BOLD,10);
	static final Font buttonFont = new Font(null,Font.BOLD,14);
	static final Font elasticityFont = new Font(null,Font.BOLD,12);
	static final Color3f controlBackColor3f = new Color3f(0.0f,0.0f,0.0f);
	static final Color controlBackColor = controlBackColor3f.get();//Color.BLACK;
	static final Color controlForeColor = new Color(0.8f,0.8f,1.0f);
	static final Color buttonForeColor = Color.BLUE;
	static final Color buttonBackColor = controlBackColor;
	static final Color canvasTextColor = new Color(0.9f,0.9f,0.9f);
	static final Color3f bugEndLightColor3f = new Color3f(0.8f,0.8f,0.8f);
	static final Color3f plasmidLightColor3f = new Color3f(0.0f,0.5f,1.0f);
	
	
	public static void setTimeStepCounts () {
		Thing.biochemCheckInt = (int)(Env.biochemDeltaT.getValue()/Env.deltaT.getValue());
		//System.out.println("biochemCheckInt = " + Thing.biochemCheckInt);
		Thing.collisionCheckInt = (int)(Env.collisionDeltaT.getValue()/Env.deltaT.getValue());
		Thing.brownianApplyInt = (int)(Env.brownianDeltaT.getValue()/Env.deltaT.getValue());
	}
	
	public static void resetElasticityTstValues () {
		Env.reportedTimeConstant = false;
		if (Env.counter == 0) { 
			curMultipleOfSeg = (int)(tstFilLengthInit/(Env.stdSegLength.getValue()*Env.parMHalfMonoSize));
		}
		tstFilLength = (curMultipleOfSeg*Env.stdSegLength.getValue()+1)*Env.parMHalfMonoSize;
		if (cantileverElasticityTst) {
			desiredDeflection = -0.01*tstFilLength;
			forceToGetDesiredDeflection = 3*EI*(desiredDeflection*1e-6)/Math.pow(tstFilLength*1e-6,3);
			expectedDeflection = (forceToGetDesiredDeflection*Math.pow(tstFilLength*1e-6,3)/(3*EI))*1e6;	// in microns.. from beam theory y=FL^3/(3EI)
		} else {
			desiredDeflection = -0.01*tstFilLength;
			forceToGetDesiredDeflection = 48*EI*(desiredDeflection*1e-6)/Math.pow(tstFilLength*1e-6,3);
			expectedDeflection = (forceToGetDesiredDeflection*Math.pow(tstFilLength*1e-6,3)/(48*EI))*1e6;	// in microns.. from beam theory y=FL^3/(48EI)
			if (secondMode) {
				expectedDeflection = 0;
				forceSeg1 = Env.curMultipleOfSeg/4 + 1;
				forceSeg2 = 3*Env.curMultipleOfSeg/4 + 1;
			} else {
				forceSeg1 = Env.curMultipleOfSeg/2 + 1;
			}
			//System.out.println ("forceSegs are " + forceSeg1 + ", " + forceSeg2);
		}
		staticForce.y = forceToGetDesiredDeflection;	// force only applied in y-direction
		staticForceR.y = -forceToGetDesiredDeflection;
		curDeltaE = initDeltaE;
		lastDistFromExpected = 1e6;
	}
	
	public static void resetPersistenceTstValues () {
		persistenceSegNumber = (int)(persistenceTstLengthInit/(Env.stdSegLength.getValue()*Env.parMHalfMonoSize)); 
		persistenceTstLength = persistenceSegNumber*Env.stdSegLength.getValue()*Env.parMHalfMonoSize;
		persistenceTrack = new ValueTracker [Env.persistenceSegNumber];
		for (int i=0;i<persistenceSegNumber;i++) {
			persistenceTrack[i] = new ValueTracker(persistenceValsToTrack,ValueTracker.DOUBLE_TYPE);
		}
		transDiffTrack.zero();
	}
	
	public static double rdm () {
		return mtRNG.raw();
	}
}
